(* Copyright (c) 2002 INRIA *)
(* Author : Gaurav Chanda , Projet CRISTAL *)

open Pxp_yacc ;
open Pxp_document ;
open Pxp_types ;
open Utils ;

value date_mini = "1900-00-01" ;
value date_mine = "1900-00-01";
value date_maxi = "3900-00-01" ;
value date_maxe = "3900-00-01" ;

class type date_s_sig = 
        object
             
             value mutable name : string ;
             value mutable v : string ;
             value mutable is_enum : bool ;
             value mutable is_fixed_maxi : bool ;
             value mutable is_fixed_maxe : bool ;
             value mutable is_fixed_mini : bool ;
             value mutable is_fixed_mine : bool ;
             value mutable enumlist : list string ;
             value mutable whitespace : string ;
             value mutable maxInclusive : string ;
             value mutable maxExclusive : string ;
             value mutable minInclusive : string ;
             value mutable minExclusive : string ;

             method get_val : unit -> string ;
             method set_obj : string -> unit ;
             method to_string : unit -> string ;
             method set_struct : string -> Pxp_document.node (Pxp_document.extension (Pxp_document.node 'a)) -> unit ;

        end ;

class date_s : date_s_sig =
        object(self)

                value mutable name = "Not_Named" ;
                value mutable v = "1999-05-31" ;
                value mutable whitespace = "collapsed" ;
                value mutable is_enum = False ; 
                value mutable is_fixed_maxi = False ;
                value mutable is_fixed_maxe = False ;
                value mutable is_fixed_mini = False ;
                value mutable is_fixed_mine = False ;
                value mutable enumlist = [] ;
                value mutable maxInclusive = date_maxi;
                value mutable minInclusive = date_mini;
                value mutable maxExclusive = date_maxe;
                value mutable minExclusive = date_mine;

                method get_val () = v ;
                method private preserve str = str ;

                method private rf ch = match ch with
                  [ '\t' -> ' '
                  | '\n' -> ' '
                  | '\r' -> ' '
                  | c -> c
                  ];
                  
                method private replace str = implode (zip self#rf (explode str));

                method  private cf ls = 
                     let l = ref ls in
                     let b = (l.val = []) in
                     let _ = if (not b) then while (List.hd l.val = ' ') do 
                             {
                                     l.val := List.tl l.val 
                             } 
                             else () in
                     let _ = l.val := List.rev l.val in
                     let b = (l.val = []) in
                     let _ = if (not b) then while (List.hd l.val = ' ') do 
                             {
                                     l.val := List.tl l.val 
                             } 
                             else () in 
                     let _ = l.val := List.rev l.val in
                     match l.val with
                      [ [] -> []
                      | [' '::ll] -> raise Bad_Argument
                      | [x::ll] -> [x::(self#cf ll)]
                      ];

                method private collapse str = 
                     let s = self#replace str in
                     let ls = self#cf (explode s) in
                     implode ls ;

                method private process_ws ws = match ws with
                  [ "preserved" -> self#preserve
                  | "replaced" -> self#replace
                  | "collapsed" -> self#collapse
                  | _ -> raise Bad_Argument
                  ];
                  
                method set_obj str = 
                     let doc = parse_wfdocument_entity default_config (from_string str) default_spec in
                     let r = doc#root in
                     let dat = (self#process_ws whitespace) r#data in
                     v := dat ;

                method to_string () =  String.concat "" ["<";name;">";v;"</";name;">"] ;
                
                method private set_attr x = match (nodename x) with
                              [ "maxInclusive" -> let r = x#attributes_as_nodes in
                                            let e1 = List.hd r in
                                            let b = nodename e1 in
                                            let v1 = if (b = "value") then e1#data
                                                      else "P0M" in
                                            let v2 = if (b = "fixed") then (e1#data = "true")
                                                     else False in
                                            let b = List.tl r in
                                            let e2 = if (not (b = [])) then
                                                    List.hd (List.tl r) else e1 in
                                            let b = (nodename e1 = nodename e2) in
                                            let v1 = if (myAnd (not b) (nodename e2 = "value"))
                                                     then e2#data else "P0M" in
                                            let v2 = if (myAnd (not b) (nodename e2 = "fixed"))
                                                     then ("true" = e2#data) else v2 in
                                            do {maxInclusive := v1; is_fixed_maxi := v2 } 
                              | "maxExclusive" -> let r = x#attributes_as_nodes in
                                            let e1 = List.hd r in
                                            let b = nodename e1 in
                                            let v1 = if (b = "value") then e1#data 
                                                      else "P0M" in
                                            let v2 = if (b = "fixed") then (e1#data = "true")
                                                     else False in
                                            let b = List.tl r in
                                            let e2 = if (not (b = [])) then
                                                    List.hd (List.tl r) else e1 in
                                            let b = (nodename e1 = nodename e2) in
                                            let v1 = if (myAnd (not b) (nodename e2 = "value"))
                                                     then e2#data else "P0M" in
                                            let v2 = if (myAnd (not b) (nodename e2 = "fixed"))
                                                     then ("true" = e2#data) else v2 in
                                            do {maxExclusive := v1; is_fixed_maxe := v2 } 
                              | "minInclusive" -> let r = x#attributes_as_nodes in
                                            let e1 = List.hd r in
                                            let b = nodename e1 in
                                            let v1 = if (b = "value") then e1#data 
                                                      else "P0M" in
                                            let v2 = if (b = "fixed") then (e1#data = "true")
                                                     else False in
                                            let b = List.tl r in
                                            let e2 = if (not (b = [])) then
                                                    List.hd (List.tl r) else e1 in
                                            let b = (nodename e1 = nodename e2) in
                                            let v1 = if (myAnd (not b) (nodename e2 = "value"))
                                                     then e2#data else "P0M" in
                                            let v2 = if (myAnd (not b) (nodename e2 = "fixed"))
                                                     then ("true" = e2#data) else v2 in
                                            do {minInclusive := v1; is_fixed_maxi := v2 } 
                              | "minExclusive" -> let r = x#attributes_as_nodes in
                                            let e1 = List.hd r in
                                            let b = nodename e1 in
                                            let v1 = if (b = "value") then e1#data 
                                                      else "P0M" in
                                            let v2 = if (b = "fixed") then (e1#data = "true")
                                                     else False in
                                            let b = List.tl r in
                                            let e2 = if (not (b = [])) then
                                                    List.hd (List.tl r) else e1 in
                                            let b = (nodename e1 = nodename e2) in
                                            let v1 = if (myAnd (not b) (nodename e2 = "value"))
                                                     then e2#data else "P0M" in
                                            let v2 = if (myAnd (not b) (nodename e2 = "fixed"))
                                                     then ("true" = e2#data) else v2 in
                                            do {minExclusive := v1; is_fixed_maxe := v2 } 
                              | "whiteSpace" -> let r = x#attributes_as_nodes in
                                                let vv = (List.hd r)#data in
                                                let b1 = not (vv = "preserve") in
                                                let b2 = not (vv = "replace") in
                                                let b3 = not (vv = "collapse") in
                                                let b = myAnd (myAnd b1 b2) b3 in
                                                if (b) then raise Bad_Argument 
                                                else whitespace := vv 
                              | "enumeration" -> let r = x#attributes_as_nodes in
                                                 let vv = (List.hd r)#data in
                                                 do {is_enum := True ; enumlist := [vv :: enumlist] }
                              | _ -> raise Bad_Attribute
                             ];

                method set_struct n sch = let alist = sch#sub_nodes in
                                          let _ = name := n in
                                          let _ = zip self#set_attr alist in
                                            ();

                (* Well Formedness of duration + Boundedness of the value not
                checked *)
       end ;

