(* Copyright (c) 2002 INRIA *)
(* Author : Gaurav Chanda , Projet CRISTAL *)

(** This module contains the description of the [decimal_s] type *)

(** This is the lower bound of the [minInclusive] attribute *)
value decimal_mini : float ;

(** This is the lower bound of the [minExclusive] attribute *)
value decimal_mine : float ;

(** This is the upper bound of the [maxInclusive] attribute *)
value decimal_maxi : float ;

(** This is the upper bound of the [maxExclusive] attribute *)
value decimal_maxe : float ;

(** This gives the general description of the [decimal_s_sig] class *)
class type decimal_s_sig = 
        object
             
             value mutable name : string ;
             (** This contains the name of the XML-element concerned *)
             
             value mutable v : float ;
             (** This contains the value of the object *)
             
             value mutable totalDigits: int ;
             (** This gives the total digits that is permissible in this type.
             The default is fixed at -1, meaning that there is no such
             restriction *)

             value mutable fractionDigits: int ;
             (** This gives the total fraction digits that is permissible in this type.
             The default is fixed at -1, meaning that there is no such
             restriction *)

             value mutable is_enum : bool ;
             (** This indicates whether the type is an enumerated type *)

             value mutable is_fixed_td : bool ;
             (** This indicates whether a derived class can have the
             [totalDigits] field different from the parent class *)

             value mutable is_fixed_fd : bool ;
             (** This indicates whether a derived class can have the
             [fractionDigits] field different from the parent class *)

             value mutable is_fixed_maxi : bool ;
             (** This indicates whether a derived class can have the
             [maxInclusive] field different from the parent class *)

             value mutable is_fixed_maxe : bool ;
             (** This indicates whether a derived class can have the
             [maxExclusive] field different from the parent class *)

             value mutable is_fixed_mini : bool ;
             (** This indicates whether a derived class can have the
             [minInclusive] field different from the parent class *)

             value mutable is_fixed_mine : bool ;
             (** This indicates whether a derived class can have the
             [minExclusive] field different from the parent class *)

             value mutable enumlist : list string ;
             (** This contains the list of entries required to define the
             enumerated type *)

             value mutable whitespace : string ;
             (** This contains information on how to handle white spaces *)
             
             value mutable maxInclusive : float ;
             (** This contains information on the maximum inclusive bound of the
             type *)

             value mutable maxExclusive : float ;
             (** This contains information on the maximum exclusive bound of the
             type *)

             value mutable minInclusive : float ;
             (** This contains information on the minimum inclusive bound of the
             type *)

             value mutable minExclusive : float ;
             (** This contains information on the minimum exclusive bound of the
             type *)


             method get_val : unit -> float ;
             (** This function returns the value held by the object *)
             
             method set_obj : string -> unit ;
             (** Sets the value of the object with the given string after making
             other attribute checks *)
             
             method to_string : unit -> string ;
             (** Returns the XML document corresponding to this object *)
             
             method set_struct : string -> Pxp_document.node (Pxp_document.extension (Pxp_document.node 'a)) -> unit ;
             (** Takes as argument, an element name and the schema that
             specifies various attributes that need to be fixed *)

        end ;

(** An instance of the [decimal_s_sig] class *)
class decimal_s : decimal_s_sig ;
