(* Copyright (c) 2002 INRIA *)
(* Author : Gaurav Chanda , Projet CRISTAL *)

(** This module defines some basic functions and exceptions that are used in
other modules *)

open Pxp_yacc ;
open Pxp_document ;
open Pxp_types ;

exception Not_Enumerated_Type;
(** This exception is raised when the enumeration constraint is not set, but an
attempt is made to make the type enumerated *)

exception Value_Not_Within_Bounds ;
(** This exception is raised when the value being assigned for a particular type
object, is not within the constraints set in that type *)

exception Not_Defined_In_Enumeration ;
(** This exception is raised when an element is assigned for an enumerated type,
but that element is not defined in the enumeration *)

exception Bad_Argument ;
(** This exception is raised when there is bad parameter passing going on in the
program *)

exception Bad_Attribute ;
(** This exception is raised when an attribute is defined a value that is not
expected *)

value myAnd : bool -> bool -> bool ;
(** This is the logical AND operation *)

value is_present : list 'a -> 'a -> bool ;
(** Searches the list for an element *)
    
value zip : ('a -> 'b) -> list 'a -> list 'b ;
(** Applies the given function on each element of a list and returns the
resultant list *)

value explode : string -> list char ;
(** Converts a string into a list of its constituent characters *)

value implode : list char -> string ;
(** Merges all the characters of a list into a string *)
    
value nodename : Pxp_document.node 'a -> string ;
(** Returns the name of the element represented by the document node *)

