(**
  \section{Remote method invocation}\label{clientapi}

   [new remote xmlrpc-server method] returns an object that can be
   used to execute the foreign method. Its [simple_call] method is
   convienience for a function that only takes one argument, and
   [zero_call] for functions without arguments. [call] must be used
   for all others. [remove] is used to remove the method from the 
   server.

   While there's nothing stopping people from using this class
   directly, using functions generated by oxridl is much nicer and
   simpler, because it handles the conversion btween XML-RPC types and
   ml types automatically, for the most part. Please do so if possible.
*)


(* \subsection{Exceptions} *)

(** This exception (Or [SoapTypes.Bad_type]) is raised if there's
technical problem with the processing -- can't connect to the server,
the server returned something besides valid XML-RPC xml, or whatever.
*)
(*exception Request_failed of string; *)
exception Request_failed ; 

(** This exception is raised if the request completes successfully, but
returned a <fault> instead of a <params> in the
<methodResponse> node. In other words, error generated from
the server because of the request, not in the transaction. It holds a
[SoapTypes.soapst] with the fault information.
*)
exception Request_fault of SoapTypes.soapst;

(* \subsection{Remote calls} *)

(** This is the string sent as the user-agent argument in HTTP headers. *)
value client_version : string;

(** This class encapsulates a remote procedure execution or removal. See above for details. *)
class remote :
  [ string ] ->
    [ string ] ->
      object
        method zero_call : unit -> SoapTypes.ty;
        method simple_call : SoapTypes.ty -> SoapTypes.ty;
        method call : list SoapTypes.ty -> SoapTypes.ty;
        method remove : unit -> SoapTypes.ty;
      end;

