(* Copyright (c) 2002 INRIA *)

open String_s ;
open Boolean_s ;
open Decimal_s ;
open Float_s ; 
open Double_s ;
open Duration_s ;
open DateTime_s ;
open Time_s ;
open Date_s ;
open GYearMonth_s ;
open GYear_s ;
open GMonthDay_s ;
open GDay_s ;
open GMonth_s ;
open HexBinary_s ;
open Base64Binary_s ;
open Uri_s ;
open Qname_s ;
open Notation_s ;
open Utils ;

type soapst = Hashtbl.t string ty
and ty = [ Unit 
         | String of string_s 
         | Boolean of boolean_s 
         | Decimal of decimal_s 
         | Float of float_s 
         | Double of double_s 
         | Duration of duration_s 
         | DateTime of dateTime_s 
         | Time of time_s 
         | Date of date_s 
         | GYearMonth of gYearMonth_s 
         | GYear of gYear_s 
         | GMonthDay of gMonthDay_s 
         | GDay of gDay_s 
         | GMonth of gMonth_s 
         | HexBinary of hexBinary_s 
         | Base64Binary of base64Binary_s 
         | Uri of uri_s 
         | Qname of qname_s 
         | Notation of notation_s 
         | Array of array ty
         | Struct of Hashtbl.t string ty
         ];

exception Bad_type ;

value make_struct () = Hashtbl.create 12 ;

value add_elem = Hashtbl.add ;
value find_elem = Hashtbl.find ;

(* Convert ML Types to SOAP - types *)

value sp_of_string s = String s ;
value sp_of_boolean b = Boolean b ;
value sp_of_decimal d = Decimal d ;
value sp_of_float f = Float f ;
value sp_of_double d = Double d ;
value sp_of_duration d = Duration d ;
value sp_of_dateTime d = DateTime d ; 
value sp_of_time t = Time t ;
value sp_of_date d = Date d ;
value sp_of_gYearMonth ym = GYearMonth ym ;
value sp_of_gYear y = GYear y ;
value sp_of_gMonthDay md = GMonthDay md ;
value sp_of_gDay d = GDay d ;
value sp_of_gMonth m = GMonth m ;
value sp_of_hexBinary hb = HexBinary hb ;
value sp_of_base64Binary bb = Base64Binary bb ;
value sp_of_uri u = Uri u ;
value sp_of_qname q = Qname q ;
value sp_of_notation n = Notation n ;
value sp_of_array ar = Array ar ;
value sp_of_struct s = Struct s ;

value skip = Unit ;

(* Convert SOAP Types to ML Types *)

value ml_of_string v = match v with
  [ String s -> s
  | _ -> raise Bad_type
  ];

value ml_of_boolean v = match v with
  [ Boolean s -> s
  | _ -> raise Bad_type
  ];

value ml_of_decimal v = match v with
  [ Decimal s -> s
  | _ -> raise Bad_type
  ];

value ml_of_float v = match v with
  [ Float s -> s
  | _ -> raise Bad_type
  ];

value ml_of_double v = match v with
  [ Double s -> s
  | _ -> raise Bad_type
  ];

value ml_of_duration v = match v with
  [ Duration s -> s
  | _ -> raise Bad_type
  ];

value ml_of_dateTime v = match v with
  [ DateTime s -> s
  | _ -> raise Bad_type
  ];

value ml_of_time v = match v with
  [ Time s -> s
  | _ -> raise Bad_type
  ];

value ml_of_date v = match v with
  [ Date s -> s
  | _ -> raise Bad_type
  ];

value ml_of_gYearMonth v = match v with
  [ GYearMonth s -> s
  | _ -> raise Bad_type
  ];

value ml_of_gYear v = match v with
  [ GYear s -> s
  | _ -> raise Bad_type
  ];

value ml_of_gMonthDay v = match v with
  [ GMonthDay s -> s
  | _ -> raise Bad_type
  ];

value ml_of_gDay v = match v with
  [ GDay s -> s
  | _ -> raise Bad_type
  ];

value ml_of_gMonth v = match v with
  [ GMonth s -> s
  | _ -> raise Bad_type
  ];

value ml_of_hexBinary v = match v with
  [ HexBinary s -> s
  | _ -> raise Bad_type
  ];

value ml_of_base64Binary v = match v with
  [ Base64Binary s -> s
  | _ -> raise Bad_type
  ];

value ml_of_uri v = match v with
  [ Uri s -> s
  | _ -> raise Bad_type
  ];

value ml_of_qname v = match v with
  [ Qname s -> s
  | _ -> raise Bad_type
  ];

value ml_of_notation v = match v with
  [ Notation s -> s
  | _ -> raise Bad_type
  ];

value ml_of_array v = match v with
  [ Array s -> s
  | _ -> raise Bad_type
  ];

value ml_of_struct v = match v with
  [ Struct s -> s
  | _ -> raise Bad_type
  ];

value ml_of_stringarray ar = match ar with
  [ Array ar -> Array.map ml_of_string ar
  | _ -> raise Bad_type 
  ];

value ml_of_booleanarray ar = match ar with
  [ Array ar -> Array.map ml_of_boolean ar
  | _-> raise Bad_type 
  ];

value ml_of_decimalarray ar = match ar with
  [ Array ar -> Array.map ml_of_decimal ar
  | _-> raise Bad_type 
  ];

value ml_of_floatarray ar = match ar with
  [ Array ar -> Array.map ml_of_float ar
  | _-> raise Bad_type 
  ];

value ml_of_doublearray ar = match ar with
  [ Array ar -> Array.map ml_of_double ar
  | _-> raise Bad_type 
  ];

value ml_of_durationarray ar = match ar with
  [ Array ar -> Array.map ml_of_duration ar
  | _-> raise Bad_type 
  ];

value ml_of_dateTimearray ar = match ar with
  [ Array ar -> Array.map ml_of_dateTime ar
  | _-> raise Bad_type 
  ];

value ml_of_timearray ar = match ar with
  [ Array ar -> Array.map ml_of_time ar
  | _-> raise Bad_type 
  ];

value ml_of_datearray ar = match ar with
  [ Array ar -> Array.map ml_of_date ar
  | _-> raise Bad_type 
  ];

value ml_of_gYearMontharray ar = match ar with
  [ Array ar -> Array.map ml_of_gYearMonth ar
  | _-> raise Bad_type 
  ];

value ml_of_gYeararray ar = match ar with
  [ Array ar -> Array.map ml_of_gYear ar
  | _-> raise Bad_type 
  ];

value ml_of_gMonthDayarray ar = match ar with
  [ Array ar -> Array.map ml_of_gMonthDay ar
  | _-> raise Bad_type 
  ];

value ml_of_gDayarray ar = match ar with
  [ Array ar -> Array.map ml_of_gDay ar
  | _-> raise Bad_type 
  ];

value ml_of_gMontharray ar = match ar with
  [ Array ar -> Array.map ml_of_gMonth ar
  | _-> raise Bad_type 
  ];

value ml_of_structarray ar = match ar with
  [ Array ar -> Array.map ml_of_struct ar
  | _-> raise Bad_type 
  ];

value parse_value sch r t = match t with
  [ "string" -> let s = new string_s in
                let _ = s#set_struct (nodename r) sch in
                let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                let _ = s#set_obj (s1) in
                String s
  | "boolean" -> let b = new boolean_s in
                 let _ = b#set_struct (nodename r) sch in
                 let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                 let _ = b#set_obj (s1) in
                 Boolean b
  | "decimal" -> let d = new decimal_s in
                 let _ = d#set_struct (nodename r) sch in
                 let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                 let _ = d#set_obj (s1) in
                 Decimal d
  | "float" -> let f = new float_s in
               let _ = f#set_struct (nodename r) sch in
               let s1 = String.concat "" ["<def>";r#data;"</def>"] in
               let _ = f#set_obj (s1) in
                 Float f
  | "double" -> let d = new double_s in
                let _ = d#set_struct (nodename r) sch in
                let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                let _ = d#set_obj (s1) in
                 Double d
  | "duration" -> let d = new duration_s in
                  let _ = d#set_struct (nodename r) sch in
                  let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                  let _ = d#set_obj (s1) in
                    Duration d
  | "dateTime" -> let d = new dateTime_s in
                  let _ = d#set_struct (nodename r) sch in
                  let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                  let _ = d#set_obj (s1) in
                    DateTime d
  | "time" -> let t = new time_s in
              let _ = t#set_struct (nodename r) sch in
              let s1 = String.concat "" ["<def>";r#data;"</def>"] in
              let _ = t#set_obj (s1) in
              Time t
  | "date" -> let d = new date_s in
              let _ = d#set_struct (nodename r) sch in
              let s1 = String.concat "" ["<def>";r#data;"</def>"] in
              let _ = d#set_obj (s1) in
              Date d
  | "gYearMonth" -> let ym = new gYearMonth_s in
                    let _ = ym#set_struct (nodename r) sch in
                    let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                    let _ = ym#set_obj (s1) in
                      GYearMonth ym
  | "gYear" -> let y = new gYear_s in
               let _ = y#set_struct (nodename r) sch in
               let s1 = String.concat "" ["<def>";r#data;"</def>"] in
               let _ = y#set_obj (s1) in
                 GYear y
  | "gMonthDay" -> let md = new gMonthDay_s in
                   let _ = md#set_struct (nodename r) sch in
                   let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                   let _ = md#set_obj (s1) in
                      GMonthDay md
  | "gDay" -> let d = new gDay_s in
              let _ = d#set_struct (nodename r) sch in
              let s1 = String.concat "" ["<def>";r#data;"</def>"] in
              let _ = d#set_obj (s1) in
                 GDay d 
  | "gMonth" -> let m = new gMonth_s in
                let _ = m#set_struct (nodename r) sch in
                let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                let _ = m#set_obj (s1) in
                 GMonth m 
  | "hexBinary" -> let hb = new hexBinary_s in
                   let _ = hb#set_struct (nodename r) sch in
                   let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                   let _ = hb#set_obj (s1) in
                      HexBinary hb
  | "base64Binary" -> let bb = new base64Binary_s in
                      let _ = bb#set_struct (nodename r) sch in
                      let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                      let _ = bb#set_obj (s1) in
                         Base64Binary bb
  | "uri" -> let u = new uri_s in
             let _ = u#set_struct (nodename r) sch in
             let s1 = String.concat "" ["<def>";r#data;"</def>"] in
             let _ = u#set_obj (s1) in
                 Uri u
  | "qname" -> let q = new qname_s in
               let _ = q#set_struct (nodename r) sch in
               let s1 = String.concat "" ["<def>";r#data;"</def>"] in
               let _ = q#set_obj (s1) in
                 Qname q  
  | "notation" -> let n = new notation_s in
                  let _ = n#set_struct (nodename r) sch in
                  let s1 = String.concat "" ["<def>";r#data;"</def>"] in
                  let _ = n#set_obj (s1) in
                        Notation n
  | _ -> raise Bad_type
  ];

value print_value v = match v with
  [ Unit -> ""
  | String s -> s#get_val()
  | Boolean b -> string_of_bool (b#get_val())
  | Decimal d -> string_of_float (d#get_val())
  | Float f -> string_of_float (f#get_val())
  | Double d -> string_of_float (d#get_val())
  | Duration d -> d#get_val()
  | DateTime d -> d#get_val()
  | Time t -> t#get_val()
  | Date d -> d#get_val()
  | GYearMonth ym -> ym#get_val()
  | GYear y -> y#get_val()
  | GMonthDay md -> md#get_val()
  | GDay d -> d#get_val()
  | GMonth m -> m#get_val()
  | HexBinary hb -> hb#get_val()
  | Base64Binary bb -> bb#get_val()
  | Uri u -> u#get_val()
  | Qname q -> q#get_val()
  | Notation n -> n#get_val()
  | _ -> "Printing not implemented"
  ];
